/*
#             (C) 2008 Hans de Goede <j.w.r.degoede@hhs.nl>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2.1 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __LIBV4L2_PRIV_H
#define __LIBV4L2_PRIV_H

#include <stdio.h>
#include <pthread.h>
#include <libv4lconvert.h> /* includes videodev2.h for us */

/* On 32 bits archs we always use mmap2, on 64 bits archs there is no mmap2 */
#ifdef __NR_mmap2
#define SYS_mmap2 __NR_mmap2
#define MMAP2_PAGE_SHIFT 12
#else
#define SYS_mmap2 SYS_mmap
#define MMAP2_PAGE_SHIFT 0
#endif

#define V4L2_MAX_DEVICES 16
/* Warning when making this larger the frame_queued and frame_mapped members of
   the v4l2_dev_info struct can no longer be a bitfield, so the code needs to
   be adjusted! */
#define V4L2_MAX_NO_FRAMES 32
#define V4L2_DEFAULT_NREADBUFFERS 4
#define V4L2_FRAME_BUF_SIZE (4096 * 4096)

#define V4L2_LOG_ERR(...) \
  do { \
    if (v4l2_log_file) { \
      fprintf(v4l2_log_file, "libv4l2: error " __VA_ARGS__); \
      fflush(v4l2_log_file); \
    } else \
      fprintf(stderr, "libv4l2: error " __VA_ARGS__); \
  } while(0)

#define V4L2_LOG_WARN(...) \
  do { \
    if (v4l2_log_file) { \
      fprintf(v4l2_log_file, "libv4l2: warning " __VA_ARGS__); \
      fflush(v4l2_log_file); \
    } else \
      fprintf(stderr, "libv4l2: warning " __VA_ARGS__); \
  } while(0)

#define V4L2_LOG(...) \
  do { \
    if (v4l2_log_file) { \
      fprintf(v4l2_log_file, "libv4l2: " __VA_ARGS__); \
      fflush(v4l2_log_file); \
    } \
  } while(0)

#define MIN(a,b) (((a)<(b))?(a):(b))

struct v4l2_dev_info {
  int fd;
  int flags;
  int open_count;
  /* actually format of the cam */
  struct v4l2_format src_fmt;
  /* fmt as seen by the application (iow after conversion) */
  struct v4l2_format dest_fmt;
  pthread_mutex_t stream_lock;
  unsigned int no_frames;
  unsigned int nreadbuffers;
  struct v4lconvert_data *convert;
  struct v4lcontrol_data *control;
  unsigned char *convert_mmap_buf;
  /* Frame bookkeeping is only done when in read or mmap-conversion mode */
  unsigned char *frame_pointers[V4L2_MAX_NO_FRAMES];
  int frame_sizes[V4L2_MAX_NO_FRAMES];
  int frame_queued; /* 1 status bit per frame */
  /* mapping tracking of our fake (converting mmap) frame buffers */
  unsigned char frame_map_count[V4L2_MAX_NO_FRAMES];
};

/* controls */
int v4l2_init_control(int fd);

/* From log.c */
void v4l2_log_ioctl(unsigned long int request, void *arg, int result);

static int v4l2_get_index(int fd);


static pthread_mutex_t v4l2_open_mutex = PTHREAD_MUTEX_INITIALIZER;
static struct v4l2_dev_info devices[V4L2_MAX_DEVICES] = { { .fd = -1 },
	{ .fd = -1 }, { .fd = -1 }, { .fd = -1 }, { .fd = -1 }, { .fd = -1 },
 { .fd = -1 }, { .fd = -1 }, { .fd = -1 }, { .fd = -1 }, { .fd = -1 },
 { .fd = -1 }, { .fd = -1 }, { .fd = -1 }, { .fd = -1 }, { .fd = -1 }};
 static int devices_used = 0;

/* From control.c */
 
#define SW_CTRL_WB_INDEX 		0
#define SW_CTRL_ROTATE_INDEX 	1
#define COUNT_SW_CTRL			2
#define CTL_EXIT				0xffff

struct v4lcontrol_data{
	 struct v4l2_queryctrl sw_ctrl[COUNT_SW_CTRL];
	 unsigned long int value[COUNT_SW_CTRL];
	 unsigned long int last_ctrl_id;
	 pthread_t event_thread;
	 int shm_data_id;
	 int shm_semaphores_id;
};

struct thread_arg{
	int fd;
	const char *device;
	struct v4lcontrol_data *control_data;
};

static struct v4lcontrol_data v4lcontrol_data_skeleton = {
	.sw_ctrl = {
		{
			.id 		= -1,
			.type 		= V4L2_CTRL_TYPE_BOOLEAN,
			.name 		= "software white balance",
			.minimum 	= 0,
			.maximum 	= 1,
			.step 		= 1,
			.default_value 	= 1
		},
  		{
			.id 		= -1,
			.type 		= V4L2_CTRL_TYPE_INTEGER,
			.name 		= "software rotate",
			.minimum 	= 0,
			.maximum 	= 270,
			.step 		= 90, 
			.default_value 	= 0
		}
	},
	.value = {
		1,
  		0
	},
	.last_ctrl_id = -1
};

void get_last_and_max_ctrl_id(int fd, long unsigned int *max,long unsigned int *last);
struct v4lcontrol_data *v4lcontrol_init(int fd, const char *device);
int v4lcontrol_destroy(struct v4lcontrol_data *control);
int v4lcontrol_queryctrl(int fd, struct v4l2_queryctrl *qctrl);
int v4lcontrol_setctrl(int fd, struct v4l2_control *ctrl);
int v4lcontrol_getctrl(int fd, struct v4l2_control *ctrl);
int binary_semaphore_down (int semid, int counterId);
int binary_semaphore_up(int semid, int counterId);
void *event_thread_func(void *arg);

#endif
